@@ -1,47 +1,39 @@
-#################################################################################
+## Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
 ##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2009
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
 ##
-##   This file is part of the R package copula.
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
 ##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
 perspCopula <- function(x, fun, n = 51, theta = -30, phi = 30, expand = 0.618, ...) {
   eps <- (.Machine$double.eps)^(1/4)
-  eps <- 0
+  eps <- 0 ## FIXME - argument with default 0 ??
   xis <- yis <- seq(0 + eps, 1 - eps, len = n)
   grids <- as.matrix(expand.grid(xis, yis))
   zmat <- matrix(fun(x, grids), n, n)
   persp(xis, yis, zmat, theta = theta, phi = phi, expand = expand, ...)
-  val <- list(x = xis, y = yis, z = zmat)
-  invisible(val)
+  invisible(list(x = xis, y = yis, z = zmat))
 }
 
 
 
 contourCopula <- function(x, fun, n = 51,...) {
   eps <- (.Machine$double.eps)^(1/4)
-  eps <- 0
+  eps <- 0 ## FIXME - argument with default 0 ??
   xis <- yis <- seq(0 + eps, 1 - eps, len = n)
   grids <- as.matrix(expand.grid(xis, yis))
   zmat <- matrix(fun(x, grids), n, n)
   contour(xis, yis, zmat, ...)
-  val <- list(x = xis, y = yis, z = zmat)
-  invisible(val)
+  invisible(list(x = xis, y = yis, z = zmat))
 }
 
 
@@ -53,21 +45,19 @@ perspMvdc <- function(x, fun,
   grids <- as.matrix(expand.grid(xis, yis))
   zmat <- matrix(fun(x, grids), nx, ny)
   persp(xis, yis, zmat, theta = theta, phi = phi, expand = expand, ...)
-  val <- list(x = xis, y = yis, z = zmat)
-  invisible(val)
+  invisible(list(x = xis, y = yis, z = zmat))
 }
 
 
 
-contourMvdc <- function(x, fun,
-                        xlim, ylim, nx = 51, ny = 51, ...) {
+contourMvdc <- function(x, fun, xlim, ylim, nx = 51, ny = 51, ...)
+{
   xis <- seq(xlim[1], xlim[2], length = nx)
   yis <- seq(ylim[1], ylim[2], length = ny)
   grids <- as.matrix(expand.grid(xis, yis))
   zmat <- matrix(fun(x, grids), nx, ny)
   contour(xis, yis, zmat, ...)
-  val <- list(x = xis, y = yis, z = zmat)
-  invisible(val)
+  invisible(list(x = xis, y = yis, z = zmat))
 }
 
 setMethod("persp", signature("indepCopula"), perspCopula)
@@ -80,10 +70,9 @@ setMethod("persp", signature("mvdc"), perspMvdc)
 setMethod("contour", signature("mvdc"), contourMvdc)
 
 
-####################################################################
-#### Graphical tools for detecting dependence
-#### Genest and Farve (2007, Journal of Hydrologic Engineering)
-####################################################################
+### Graphical tools for detecting dependence
+### Genest and Farve (2007, Journal of Hydrologic Engineering)
+
 ChiPlot <- function(x, plot=TRUE, pval = 0.95, ...) {
 #### originally proposed by Fisher and Switzer (1985 Biometrika, 2001 Am. Stat.)
   ## x is a n by 2 matrix
@@ -130,7 +119,7 @@ KPlot <- function(x, plot=TRUE, ...) {
               function(i) integrate(integrand, 0, 1, i = i,
                                     rel.tol=.Machine$double.eps^0.25)$value)
   W <- n * choose(n - 1, 1:n - 1) * W
-  
+
   if (plot) {
     plot(W, H, xlim=c(0, 1), ylim=c(0, 1))
     curve(K0(x), add=TRUE, col="blue")
@@ -144,3 +133,44 @@ KPlot <- function(x, plot=TRUE, ...) {
 ## ChiPlot(cbind(x, y))
 ## KPlot(cbind(x, y))
 
+################################################################################
+
+
+##' @title A scatter plot matrix with nice variable names
+##' @param data numeric matrix or as.matrix(.)able
+##' @param varnames variable names, typically unspecified
+##' @param Vname character string to become "root variable name"
+##' @param col.mat matrix of colors
+##' @param bg.col.mat matrix of background colors
+##' @param ... further arguments to splom()
+##' @return a splom() object
+##' @author Martin Maechler
+splom2 <- function(data, varnames=NULL, Vname="U", xlab="",
+                   col.mat=NULL, bg.col.mat=NULL, ...)
+{
+    stopifnot(require(lattice),
+	      is.numeric(data <- as.matrix(data)),
+	      (d <- ncol(data)) >= 1)
+    if(is.null(varnames)) {
+	varnames <- do.call(expression,
+			    lapply(1:d, function(i)
+				   substitute(italic(A[I]), list(A = as.name(Vname), I=0+i))))
+    }
+    n <- nrow(data)
+    if(is.null(col.mat))
+        col.mat <- matrix(trellis.par.get("plot.symbol")$col, n,d)
+    if(is.null(bg.col.mat))
+        bg.col.mat <- matrix(trellis.par.get("background")$col, n,d)
+    ## From Deepayan Sarkar, working around missing feature
+    ##		(which should be in next release) of lattice
+    my.diag.panel <- function(x, varname, ...)
+        diag.panel.splom(x, varname=parse(text=varname), ...)
+    ## splom
+    splom(~data[,1:d], varnames=varnames, diag.panel=my.diag.panel, xlab="",
+          panel = function(x, y, i, j, ...) {
+              panel.fill(bg.col.mat[i,j])
+              panel.splom(x, y, col=col.mat[i,j], ...)
+          }, ...)
+}
+
+
